clear;clc;
%% Set up the domain data structures
n = 4000;
x_a = 0; x_b = 1; % The beginning and end of the Cartesian grid
h = (x_b - x_a)/(n-1);

m =  4; % Gram Polynomial max order
d =  5; % Number of orthogonal polynomial matching points
C = 25; % Number of continuation points

% Load the precomputed FC data
load(['FC_data_d', num2str(d), '.mat']);

% The grid we have data on and wish to interpolate/compute derivatives/solve DEs
x = (x_a:h:x_b).';
% The interior grid where the ODE is solved, as opposed to the full grid where the
% boundary conditions are also enforced.
x_int = x(2:end-1);
% The grid of continuation points (for plotting purposes)
x_cont = x_b + h*(1:C).';

% Period of our FC interpolating Fourier Series -- note the extended period
fourPts = n + C;
prd = fourPts*h;
if (mod(fourPts, 2) == 0)
    k = transpose([0:fourPts/2, -fourPts/2+1:-1]);
else
    k = transpose([0:(fourPts-1)/2, -(fourPts-1)/2:-1]);
end

% GMRES solver
%% Evaluate the function and perform continuations
a = 5.4*pi;
b = 2.7*pi;
c = 2*pi;
f = @(y) (1 + a^2*sin(a*y - b) - c^2*cos(c*y) - (a*cos(a*y - b) + c*sin(c*y)).^2) .* exp(sin(a*y - b) - cos(c*y));
usolfcn = @(y) exp(sin(a*y - b) - cos(c*y));
p = zeros(n-2, 1);
q = ones(n-2, 1);

%p = 2*ones(length(x_int), 1);
%q = -1*ones(length(x_int),1);
%f = @(y) y.*exp(y) + 4;
%usolfcn = @(y) exp(y).*y + exp(y) + exp(y).*y.^3 / 6 + 4;

%pfcn = @(y) y;
%qfcn = @(y) -y.^2;
%p = pfcn(x_int);
%q = qfcn(x_int);
%f = @(y) 0*y;
%usolfcn = @(y) y + y.*log(y);

usol = usolfcn(x_int);

B_l = usolfcn(x_a);
B_r = usolfcn(x_b);
fx = f(x_int);

gmresParam.rstrt = 10;
gmresParam.tol = 10^-14;
gmresParam.maxit = 1000;
[u,flag,relres,iter] = fc_ode_gmres(p, q, fx, B_l, B_r, gmresParam, h, k, d, C, Q, A);
if (flag == 0)
    fprintf('GMRES converged in (%d,%d) (outer,inner) iterations, relative residual %1.3e\n',...
        iter(1), iter(2), relres);
elseif (flag == 1)
    fprintf('GMRES failed to converge\n');
elseif (flag == 2)
    fprintf('Preconditioner is ill-conditioned\n');
else
    fprintf('GMRES stagnated at iteration (%d,%d) (outer,inner), relative residual %1.3e\n',...
        iter(1), iter(2), relres);
end

figure(1);
plot(x_int, usol, 'b-o', x_int, u, 'r-');
xlabel('x');
ylabel('u');
title('FC-ODE vs. exact solution');
legend('Exact ODE soln', ['FC-ODE (n= ', num2str(n), '), FD-Precond']);

figure(2);
usol_err = abs(usol - u);
plot(x_int, usol_err, 'b-*');
xlabel('x');
ylabel('|u - u_e|');
title('FC-ODE error');
