clear;
%% Set up the domain data structures
n = 320; % Number of discretization points
x_a = 0; x_b = 1; % The beginning and end of the Cartesian grid

h = (x_b - x_a)/(n-1);

m =  4; % Gram Polynomial max order
d =  5; % Number of orthogonal polynomial matching points
C = 25; % Number of continuation points

% Load the precomputed FC data
load(['FC_data_d', num2str(d), '.mat']);

% The grid we have data on and wish to interpolate/compute derivatives/solve DEs
x = (x_a:h:x_b).';
% The grid of continuation points (for plotting purposes)
x_cont = x_b + h*(1:C).';

% Period of our FC interpolating Fourier Series -- note the extended period
fourPts = n + C;
prd = fourPts*h;
if (mod(fourPts, 2) == 0)
    k = transpose([0:fourPts/2, -fourPts/2+1:-1]);
else
    k = transpose([0:(fourPts-1)/2, -(fourPts-1)/2:-1]);
end

%% Evaluate the function and perform continuations
%f = @(y) y.^2 - 1*y + 1;
%fder = @(y) 2*y - 1;
%f2der = @(y) 2;

a = 5.4*pi;
b = 2.7*pi;
c = 2*pi;
f = @(y) exp(sin(a*y - b) - cos(c*y));
fder = @(y) (a*cos(a*y - b) + c*sin(c*y)).*exp(sin(a*y - b) - cos(c*y));
f2der = @(y) -(a^2*sin(a*y - b) - c^2*cos(c*y) - (a*cos(a*y - b) + c*sin(c*y)).^2) .* exp(sin(a*y - b) - cos(c*y));

fx = f(x);
filter = 0;
[fx_cont_coeffs fc fc_l fc_r] = fcont_gram_blend(fx, d, C, Q, A);

%% Plot Results
figure(1);
plot(x, fx, 'b*', x_cont, fc(n+1:end), 'rd');
xlabel('x');
ylabel('f(x)');
title ('Fourier continuation')

z = linspace(x_a, x_b, 8000);
fc_fine_grid = real(exp(2*pi*i*(z - x_a).'*k.'/prd) * ...
        fx_cont_coeffs(:));
ferr = abs(fc_fine_grid - f(z).');
fprintf('||f(x)||: %1.3e\n', max(ferr));


figure(2)
plot(z, log10(ferr));
xlabel('x', 'Fontsize', 15);
ylabel('log_1_0(e)', 'Fontsize', 15);
title ('Pointwise error')



